#!/bin/bash
###
### This file is covered by the GNU General Public License
### version 3 or later.
###
### Copyright (C) 2021-2024, ALT Linux Team

############################
### ftp protocol handler ###
############################

# Outputs a list of additional required tools
#
get_proto_requires()
{
	echo "curl"
}

# Returns 0 if write operations are allowed on the remote side
#
have_write_access()
{
	[ -n "$remote_user" ] && [ -n "$remote_pass" ]
}

# Sends an FTP request and captures the FTP response header
#
grab_ftp_header()
{
	local opts url="$1"
	local status i=0 rc=0
	local tmpf timeout=30 dir=
	local regex="s/^Content\-Length:\s+([0-9]+)/\1/p"

	if [ "$url" = "-d" ] && [ -n "${2-}" ]; then
		url="$2"
		dir=1
	fi

	opts="-sI --no-buffer --connect-timeout 5 --max-time 7"
	opts="$opts --max-redirs 2 --write-out %{http_code} --ftp-pasv"
	[ -z "$remote_user" ] ||
		opts="$opts -u \"${remote_user//\"/\\\"}:${remote_pass//\"/\\\"}\""
	tmpf="$(run mktemp -qt)"
	filesize=

	while [ "$i" -lt "$timeout" ]; do
		rc=0; run rm -f -- "$tmpf"
		status="$(run curl $opts --output "$tmpf" --url "$url")" ||
			rc="$?"
		[ ! -s "$tmpf" ] ||
			fdump "$tmpf"
		dbg "curl status: %s, protocol status: %s" "$rc" "$status"

		case "$status" in
		202|225|231|331|332|404|421|425|426|450|451|5??)
			# This is unrecoverable client/server error,
			# there is no point in waiting for a fix
			log "unrecoverable ftp error: %s" "$status"
			rc=1
			break
			;;

		250)	# If it is a directory, then OK
			[ -n "$dir" ] && rc=0 ||
				rc=1
			break
			;;

		200|226|350)
			# OK
			if [ "$rc" = 0 ]; then
				filesize="$(sed -n -E -- "$regex" "$tmpf" |
						tail -n1 |sed -e 's,\r$,,')"
				[ -z "$filesize" ] || break
			fi
			rc=1
			;;

		*)	rc=1
			;;
		esac

		i=$((3 + $i))
		sleep 3
	done

	run rm -f -- "$tmpf"

	return "$rc"
}

# Returns 0 if the specified directory exists on the remote side
#
dir_exists()
{
	local filesize=

	grab_ftp_header -d "$backup/$1/" && [ -z "$filesize" ]
}

# Returns 0 if the specified file on the remote side is not empty
#
file_exists()
{
	local filesize=

	grab_ftp_header "$backup/$1" && [ -n "$filesize" ] &&
		is_number "$filesize" && [ "$filesize" -gt 0 ]
}

# Displays the specified file size (in bytes) if it exists on the remote side
#
get_file_size()
{
	local filesize=

	if grab_ftp_header "$backup/$1"; then
		printf "%s" "$filesize"
	fi
}

# Creates the specified directory on the remote side
#
create_directory()
{
	# run mkdir -p -m 0700 $verbose -- "$backup/$1"
	: TODO...
}

# Reads the specified file on the remote side and writes it to stdout
#
read_file()
{
	local opts filesize="${2-}"

	opts="--silent --no-buffer --connect-timeout 5 --max-redirs 2 --ftp-pasv"
	[ -z "$filesize" ] ||
		opts="$opts --max-filesize $filesize"
	[ -z "$remote_user" ] ||
		opts="$opts -u \"${remote_user//\"/\\\"}:${remote_pass//\"/\\\"}\""
	curl $opts -- "$backup/$1" || printf "%s, curl\n" "$?" >"$workdir"/ERROR
}

# Writes to the specified file on the remote side, reads it from stdin
#
write_file()
{
	# cat >"$backup/$1"
	: TODO...
}

