#!/bin/bash
###
### This file is covered by the GNU General Public License
### version 3 or later.
###
### Copyright (C) 2021-2024, ALT Linux Team

###########################################
### Helper script running inside chroot ###
###########################################

# Safety first
set -o errexit
set -o noglob
set -o nounset
set -o errtrace

# Working directory
[ -d "${TMPDIR-}" ] ||
	TMPDIR="$(realpath -- "${0%/*}")"
export TMPDIR
progname=

# Bootstrap
. "$TMPDIR"/defaults.sh
. "$TMPDIR"/internal.sh
. "$TMPDIR"/common.sh
. "$TMPDIR"/logger.sh

# User-defined parameters
[ ! -s "$TMPDIR"/sysrest.ini ] ||
	. "$TMPDIR"/sysrest.ini

# Chroot parameters
. "$TMPDIR"/chroot.ini

# Catch all unexpected errors
trap 'unexpected_error "${BASH_SOURCE[0]##*/}" "$LINENO"' ERR


A_msg()
{
	msg "The grub-efi bootloader for %s is already installed!" "$platform"
}

BIOS_install()
{
	msg "Installing grub-pc for Legacy/CSM boot mode..."
	run grub-install --target=i386-pc --recheck $verbose \
			 --boot-directory=/boot -- "$target"
}

ESP_install()
{
	local tgt="$1"

	msg "Installing grub-efi for UEFI boot mode..."
	run grub-install --target="$tgt" --recheck $verbose   \
			 --efi-directory=/boot/efi --no-nvram \
			 --boot-directory=/boot -- "$target"
}

UEFI_install()
{
	local tgt="$1"

	if [ -n "${HAVE_ESP-}" ]; then
		if [ -s /boot/grub/"$tgt"/core.efi ] &&
		   [ -s /boot/efi/EFI/"$bootldr_id"/grub.cfg ]
		then
			A_msg
		else
			ESP_install "$tgt"
		fi
	fi
}

PReP_install()
{
	msg "Installing grub-ieee1275 for IBM Power..."
	run grub-install --target=powerpc-ieee1275 $verbose \
			 --recheck${no_nvram:+ --no-nvram}  \
			 --boot-directory=/boot -- "$target"
}

track_hook()
{
	local rc=0 hook="$1"

	[ -s "$TMPDIR/$hook.sh" ] ||
		return 0
	log "Entering hook '%s'..." "$hook"
	. "$TMPDIR/$hook.sh" || rc="$?"
	log "Leaving hook '%s'..." "$hook"

	return "$rc"
}

chroot_exit_handler()
{
	local rc="$?"

	trap - EXIT; cd /
	msg "Unmounting pseudo fs..."
	run umount $quiet $verbose -- /dev/pts ||
		run umount -fl $quiet $verbose -- /dev/pts
	run umount $quiet $verbose -- /sys ||
		run umount -fl $quiet $verbose -- /sys
	run umount $quiet $verbose -- /proc ||
		run umount -fl $quiet $verbose -- /proc
	exit "$rc"
}


# Entry point
msg "Mounting pseudo fs..."
run mount -t proc $verbose -- none /proc
run mount -t sysfs $verbose -- none /sys
run mount -t devpts $verbose -- none /dev/pts

# Setup an exit handler
trap chroot_exit_handler EXIT

# Check the target platform
detect_hw_platform

# Use user-defined hook
track_hook chroot-pre

# Hardware specialization
for flavour in ${kernel_flavours-} _; do
	[ -d /lib/modules ] && [ -d /boot ] ||
		break
	[ "$flavour" != _ ] ||
		continue

	for kver in $(run ls -1 /lib/modules/ |sort -Vr) _; do
		[ -z "${kver##*-$flavour-*}" ] ||
			continue
		[ -s "/lib/modules/$kver/modules.alias" ] ||
			continue
		run make-initrd $verbose --kernel="$kver"
		[ -e /boot/initrd.img ] ||
			run ln -snf $verbose -- \
				"initrd-$kver.img" /boot/initrd.img
		[ -e /boot/"initrd-$flavour.img" ] ||
			run ln -snf $verbose -- \
				"initrd-$kver.img" /boot/"initrd-$flavour.img"
	done

	unset kver
done
unset flavour

# Install bootloader
case "$platform" in
aarch64)
	UEFI_install arm64-efi
	run update-grub
	;;
i586)
	[ -z "${HAVE_BIOS-}" ] ||
		BIOS_install
	run update-grub
	;;
ppc64le)
	[ -z "${HAVE_PReP-}" ] ||
		PReP_install
	run update-grub
	;;
x86_64)
	[ -z "${HAVE_BIOS-}" ] ||
		BIOS_install
	UEFI_install x86_64-efi
	run update-grub
	;;
esac

# Remove old and generate new SSH host keys
if [ -d /etc/openssh ] && [ -z "${keep_ssh_host_keys-}" ]; then
	if have_binary find && have_binary ssh-keygen; then
		run find /etc/openssh -maxdepth 1 -type f \
			-name 'ssh_host_*_key.pub' -or    \
			-name 'ssh_host_*_key' -delete ||:
		run ssh-keygen -A${quiet:+ -q}${verbose:+ -vvv}
	fi
fi

# Remove "Suspend" button
[ -z "${no_suspend-}" ] ||
	run systemctl $quiet mask suspend.target

# Remove "Hibernate" button
[ -z "${no_hibernate-}" ] ||
	run systemctl $quiet mask hibernate.target

# Use user-defined hook
track_hook chroot-post

